@@ -23,6 +23,8 @@
 #ifndef __DH_GECKO_UTILS_H__
 #define __DH_GECKO_UTILS_H__
 
+#include <gtkmozembed.h>
+
 G_BEGIN_DECLS
 
 enum {
@@ -30,17 +32,12 @@ enum {
 	DH_GECKO_PREF_FONT_FIXED
 };
 
-void            dh_gecko_utils_set_font      (gint          font_type,
-					      const gchar  *fontname);
-void            dh_gecko_utils_init_services (void);
-
-/*
-gboolean        dh_gecko_find           (GtkMozEmbed  *html,
-					 const gchar  *str,
-					 gboolean      match_case,
-					 gboolean      wrap,
-					 gboolean      forward);
-*/
+void dh_gecko_utils_set_font                  (gint         font_type,
+					       const gchar *fontname);
+void dh_gecko_utils_init_services             (void);
+gint dh_gecko_utils_get_mouse_event_button    (gpointer     event);
+gint dh_gecko_utils_get_mouse_event_modifiers (gpointer     event);
+void dh_gecko_utils_copy_selection            (GtkMozEmbed *embed);
 
 G_END_DECLS
 
