@@ -2,6 +2,7 @@
 /*
  * Copyright (C) 2001-2003 CodeFactory AB
  * Copyright (C) 2001-2003 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2005      Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +22,7 @@
 
 #include <config.h>
 #include <string.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkaccessible.h>
 #include <gtk/gtkcellrenderertext.h>
@@ -32,7 +34,7 @@
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtktreeselection.h>
-#include <libgnome/gnome-i18n.h>
+
 
 #include "dh-marshal.h"
 #include "dh-keyword-model.h"
@@ -43,6 +45,8 @@
 struct _DhSearchPriv {
 	DhKeywordModel *model;
 
+	DhLink         *selected_link;
+	
 	GtkWidget      *entry;
 	GtkWidget      *hitlist;
 
@@ -156,6 +160,8 @@ search_init (DhSearch *search)
 	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->hitlist),
 				 GTK_TREE_MODEL (priv->model));
 
+	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (priv->hitlist), FALSE);
+
 	gtk_box_set_spacing (GTK_BOX (search), 2);
 }
 
@@ -173,9 +179,6 @@ search_selection_changed_cb (GtkTreeSelection *selection, DhSearch *search)
 	DhSearchPriv *priv;
  	GtkTreeIter   iter;
 	
-	g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
-	g_return_if_fail (DH_IS_SEARCH (search));
-
 	priv = search->priv;
 
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
@@ -185,22 +188,29 @@ search_selection_changed_cb (GtkTreeSelection *selection, DhSearch *search)
 				    DH_KEYWORD_MODEL_COL_LINK, &link,
 				    -1);
 
- 		d(g_print ("Emiting signal with link to: %s (%s)\n",
-			   link->name, link->uri));
+		if (link != priv->selected_link) {
+			priv->selected_link = link;
 		
-		g_signal_emit (search, signals[LINK_SELECTED], 0, link);
+			d(g_print ("Emiting signal with link to: %s (%s)\n",
+				   link->name, link->uri));
+		
+			g_signal_emit (search, signals[LINK_SELECTED], 0, link);
+		}
 	}
 }
 
+/* Make it possible to jump back to the currently selected item, useful when the
+ * html view has been scrolled away.
+ */
 static gboolean
 search_tree_button_press_cb (GtkTreeView    *view,
 			     GdkEventButton *event,
 			     DhSearch       *search)
 {
-	GtkTreePath  *path = NULL;
+	GtkTreePath  *path;
 	GtkTreeIter   iter;
 	DhSearchPriv *priv;
-	DhLink       *link = NULL;
+	DhLink       *link;
 
 	priv = search->priv;
 	
@@ -217,6 +227,8 @@ search_tree_button_press_cb (GtkTreeView    *view,
 			    &iter,
 			    DH_KEYWORD_MODEL_COL_LINK, &link,
 			    -1);
+
+	priv->selected_link = link;
 	
 	g_signal_emit (search, signals[LINK_SELECTED], 0, link);
 	
@@ -279,9 +291,6 @@ search_entry_changed_cb (GtkEntry *entry, DhSearch *search)
 {
 	DhSearchPriv *priv;
 	
-	g_return_if_fail (GTK_IS_ENTRY (entry));
-	g_return_if_fail (DH_IS_SEARCH (search));
-	
 	priv = search->priv;
  
 	d(g_print ("Entry changed\n"));
@@ -298,9 +307,6 @@ search_entry_activated_cb (GtkEntry *entry, DhSearch *search)
 	DhSearchPriv *priv;
 	gchar             *str;
 	
-	g_return_if_fail (GTK_IS_ENTRY (entry));
-	g_return_if_fail (DH_IS_SEARCH (search));
-
 	priv = search->priv;
 	
 	str = (gchar *) gtk_entry_get_text (GTK_ENTRY (priv->entry));
@@ -317,8 +323,6 @@ search_entry_text_inserted_cb (GtkEntry    *entry,
 {
 	DhSearchPriv *priv;
 	
- 	g_return_if_fail (DH_IS_SEARCH (search));
-	
 	priv = search->priv;
 	
 	if (!priv->idle_complete) {
@@ -337,8 +341,6 @@ search_complete_idle (DhSearch *search)
 	GList             *list;
 	gint               text_length;
 	
-	g_return_val_if_fail (DH_IS_SEARCH (search), FALSE);
-	
 	priv = search->priv;
 	
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
@@ -369,8 +371,6 @@ search_filter_idle (DhSearch *search)
 	gchar        *str;
 	DhLink       *link;
 	
-	g_return_val_if_fail (DH_IS_SEARCH (search), FALSE);
-
 	priv = search->priv;
 
 	d(g_print ("Filter idle\n"));
@@ -393,17 +393,17 @@ search_complete_func (DhLink *link)
 	return link->name;
 }
 
-
 GtkWidget *
 dh_search_new (GList *keywords)
 {
-	DhSearch          *search;
-	DhSearchPriv      *priv;
-	GtkTreeSelection  *selection;
-        GtkWidget         *list_sw;
-	GtkWidget         *frame;
-	GtkWidget         *hbox;
-	GtkWidget         *label;
+	DhSearch         *search;
+	DhSearchPriv     *priv;
+	GtkTreeSelection *selection;
+        GtkWidget        *list_sw;
+	GtkWidget        *frame;
+	GtkWidget        *hbox;
+	GtkWidget        *label;
+	GtkCellRenderer  *cell;
 		
 	search = g_object_new (DH_TYPE_SEARCH, NULL);
 
@@ -416,12 +416,8 @@ dh_search_new (GList *keywords)
 	
 	label = gtk_label_new_with_mnemonic (_("_Search:"));
 
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4);
-
 	priv->entry = gtk_entry_new ();
 
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), priv->entry);
-
 	g_signal_connect (priv->entry, "key_press_event",
 			  G_CALLBACK (search_entry_key_press_event_cb),
 			  search);
@@ -451,22 +447,27 @@ dh_search_new (GList *keywords)
 	
         list_sw = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (list_sw),
-                                        GTK_POLICY_AUTOMATIC, 
+                                        GTK_POLICY_NEVER, 
                                         GTK_POLICY_AUTOMATIC);
 
 	gtk_container_add (GTK_CONTAINER (frame), list_sw);
+
+	cell = gtk_cell_renderer_text_new ();
+	g_object_set (cell,
+		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      NULL);
 	
 	gtk_tree_view_insert_column_with_attributes (
 		GTK_TREE_VIEW (priv->hitlist), -1,
-		_("Section"), gtk_cell_renderer_text_new (),
+		_("Section"), cell,
 		"text", 0,
 		NULL);
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (priv->hitlist),
 					   FALSE);
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (priv->hitlist), FALSE);
 
-	selection = gtk_tree_view_get_selection (
-		GTK_TREE_VIEW (priv->hitlist));
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->hitlist));
 
 	g_signal_connect (selection, "changed",
 			  G_CALLBACK (search_selection_changed_cb),
